@@ -32,27 +32,9 @@ pub enum Error {
     /// Error creating root dir for client
     #[error("Error creating root dir for client")]
     CouldNotCreateRootDir,
-    /// Signature Aggregation Error
-    #[error("Error on aggregating signatures from network")]
-    Aggregation(String),
-    /// Asymmetric Key Decryption Failed.
-    #[error("Asymmetric key decryption failure")]
-    AsymmetricDecipherFailure,
-    /// Symmetric Key Decryption Failed.
-    #[error("Symmetric key decryption failure")]
-    SymmetricDecipherFailure,
-    /// Received unexpected data.
-    #[error("Unexpected data received")]
-    ReceivedUnexpectedData,
     /// Received unexpected event.
     #[error("Unexpected event received")]
     ReceivedUnexpectedEvent,
-    /// Could not query elder.
-    #[error("Problem querying elder")]
-    ElderQuery,
-    /// Could not connect to elder.
-    #[error("Problem connecting to elder")]
-    ElderConnection,
     /// Client has not gone through qp2p bootstrap process yet
     #[error("Client has not yet acquired any network knowledge, so anything sent is guaranteed to have a lengthy AE process")]
     NoNetworkKnowledge,
@@ -64,9 +46,6 @@ pub enum Error {
         "Problem connecting to sufficient elders. A supermajority of responses is unobtainable. {0} were connected to, {1} needed."
     )]
     InsufficientElderConnections(usize, usize),
-    /// Could not query elder.
-    #[error("Problem receiving query via qp2p")]
-    ReceivingQuery,
     /// Cannot store empty bytes..
     #[error("Cannot store empty bytes.")]
     EmptyBytesProvided,
@@ -79,51 +58,21 @@ pub enum Error {
     /// The provided bytes is too large to be a `Spot`.
     #[error("The provided bytes is too large to be a `Spot`")]
     TooLargeToBeSpot,
-    /// Could not send query to elder.
-    #[error("Problem sending query via qp2p")]
-    SendingQuery,
     /// No query response before timeout
     #[error("Query timed out")]
     QueryTimedOut,
-    /// Could not query elder.
-    #[error("Problem receiving query internally in sn_client")]
-    QueryReceiverError,
     /// Could not get an encryption object.
     #[error("Could not get an encryption object.")]
     NoEncryptionObject,
     /// Could not query elder.
     #[error("Failed to obtain any response")]
     NoResponse,
-    /// No BLS section key known.
-    #[error("No BLS Section Key available")]
-    NoBlsSectionKey,
-    /// No section prefix found for session
-    #[error("We do not have a section prefix.")]
-    NoSectionPrefixKnown,
-    /// Unexpected message type received while joining.
-    #[error("Unexpected message type receivied while joining: {0}")]
-    UnexpectedMessageOnJoin(String),
-    /// Permission set provided is not a PublicPermissionSet.
-    #[error("Expected public permission set")]
-    NotPublicPermissions,
-    /// Permission set provided is not a PrivatePermissionSet.
-    #[error("Expected private permission set")]
-    NotPrivatePermissions,
-    /// Did not receive an incoming connection listener from qp2p
-    #[error("Could not listen on elder connection")]
-    NoElderListenerEstablished,
-    /// Incorrect user permissions were returned
-    #[error("Incorrect user permissions were returned")]
-    IncorrectPermissions,
     /// No operation Id could be found
     #[error("Could not retrieve the operation id of a query response")]
     UnknownOperationId,
     /// Unexpected response received
     #[error("Unexpected response received when querying {0:?}")]
     UnexpectedQueryResponse(QueryResponse),
-    /// Not in testnet "simulated payout" mode
-    #[error("Simulated payouts unavailable without 'simualted-payouts' feature flag at build")]
-    NotBuiltWithSimulatedPayouts,
     /// Other types errors
     #[error(transparent)]
     NetworkDataError(#[from] DtError),
@@ -160,12 +109,6 @@ pub enum Error {
     /// Bincode error
     #[error(transparent)]
     Serialisation(#[from] Box<bincode::ErrorKind>),
-    /// Sled error.
-    #[error("Sled error:: {0}")]
-    Sled(#[from] sled::Error),
-    /// Database error.
-    #[error("Database error:: {0}")]
-    Database(#[from] crate::dbs::Error),
     /// Could not retrieve all chunks required to decrypt the data. (Expected, Actual)
     #[error("Not enough chunks! Required {}, but we have {}.)", _0, _1)]
     NotEnoughChunks(usize, usize),
