@@ -6,8 +6,8 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use super::{connected_peers::ConnectedPeers, msg_count::MsgCount, BackPressure};
-use crate::messaging::{system::LoadReport, WireMsg};
+use super::{msg_count::MsgCount, BackPressure};
+use crate::messaging::{system::LoadReport, MessageId, WireMsg};
 use crate::routing::{
     error::{Error, Result},
     log_markers::LogMarker,
@@ -15,14 +15,13 @@ use crate::routing::{
 };
 use bytes::Bytes;
 use futures::{
-    future::{FutureExt, TryFutureExt},
+    future::TryFutureExt,
     stream::{FuturesUnordered, StreamExt},
 };
 use qp2p::Endpoint;
 use std::{future, net::SocketAddr};
 use tokio::{sync::mpsc, task};
 use tracing::Instrument;
-use xor_name::XorName;
 
 // Communication component of the node to interact with other nodes.
 #[derive(Clone)]
@@ -31,7 +30,6 @@ pub(crate) struct Comm {
     event_tx: mpsc::Sender<ConnectionEvent>,
     msg_count: MsgCount,
     back_pressure: BackPressure,
-    connected_peers: ConnectedPeers,
 }
 
 impl Drop for Comm {
@@ -57,16 +55,10 @@ impl Comm {
             Endpoint::new(local_addr, Default::default(), config).await?;
 
         let msg_count = MsgCount::new();
-        let connected_peers = ConnectedPeers::default();
 
         let _handle = task::spawn(
-            handle_incoming_connections(
-                incoming_connections,
-                event_tx.clone(),
-                msg_count.clone(),
-                connected_peers.clone(),
-            )
-            .in_current_span(),
+            handle_incoming_connections(incoming_connections, event_tx.clone(), msg_count.clone())
+                .in_current_span(),
         );
 
         let back_pressure = BackPressure::new();
@@ -76,7 +68,6 @@ impl Comm {
             event_tx,
             msg_count,
             back_pressure,
-            connected_peers,
         })
     }
 
@@ -94,16 +85,10 @@ impl Comm {
         let (bootstrap_peer, peer_incoming) = bootstrap_peer.ok_or(Error::BootstrapFailed)?;
 
         let msg_count = MsgCount::new();
-        let connected_peers = ConnectedPeers::default();
 
         let _handle = task::spawn(
-            handle_incoming_connections(
-                incoming_connections,
-                event_tx.clone(),
-                msg_count.clone(),
-                connected_peers.clone(),
-            )
-            .in_current_span(),
+            handle_incoming_connections(incoming_connections, event_tx.clone(), msg_count.clone())
+                .in_current_span(),
         );
 
         let _ = task::spawn(
@@ -112,7 +97,6 @@ impl Comm {
                 peer_incoming,
                 event_tx.clone(),
                 msg_count.clone(),
-                connected_peers.clone(),
             )
             .in_current_span(),
         );
@@ -123,7 +107,6 @@ impl Comm {
                 event_tx,
                 msg_count,
                 back_pressure: BackPressure::new(),
-                connected_peers,
             },
             UnnamedPeer::connected(bootstrap_peer),
         ))
@@ -133,12 +116,6 @@ impl Comm {
         self.endpoint.public_addr()
     }
 
-    /// Get the SocketAddr of a connection using the connection ID (XorName)
-    pub(crate) async fn get_peer_address(&self, connection_id: &XorName) -> Option<SocketAddr> {
-        let peer = self.connected_peers.get_by_id(connection_id).await?;
-        Some(peer.address())
-    }
-
     /// Sends a message on an existing connection. If no such connection exists, returns an error.
     pub(crate) async fn send_on_existing_connection(
         &self,
@@ -159,11 +136,7 @@ impl Comm {
             let connection = if let Some(connection) = recipient.connection().await {
                 Ok(connection)
             } else {
-                self.connected_peers
-                    .get_by_address(&addr)
-                    .map(|res| res.ok_or(None))
-                    .map_ok(|client| client.connection().clone())
-                    .await
+                Err(None)
             };
 
             future::ready(connection)
@@ -264,127 +237,56 @@ impl Comm {
         // Run all the sends concurrently (using `FuturesUnordered`). If any of them fails, pick
         // the next recipient and try to send to them. Proceed until the needed number of sends
         // succeeds or if there are no more recipients to pick.
-        let send = |recipient: &'r Peer, msg_bytes: Bytes, force_reconnection: bool| {
-            async move {
-                trace!(
-                    "Sending message ({} bytes, msg_id: {:?}) to {} of delivery group size {}",
-                    msg_bytes.len(),
-                    msg_id,
-                    recipient,
-                    delivery_group_size,
-                );
-
-                let retries = self.back_pressure.get(&recipient.addr()).await; // TODO: more laid back retries with lower priority, more aggressive with higher
-
-                let (connection, reused) = if let Some(connection) = {
-                    if force_reconnection {
-                        None
-                    } else {
-                        recipient.connection().await
-                    }
-                } {
-                    trace!(
-                        connection_id = connection.id(),
-                        src = %connection.remote_address(),
-                        "{}",
-                        LogMarker::ConnectionReused
-                    );
-                    (Ok(connection), true)
-                } else if let Some(connection) = {
-                    let existing_connection =
-                        self.connected_peers.get_by_address(&recipient.addr()).await;
-                    (!force_reconnection).then(|| existing_connection).flatten()
-                } {
-                    let connection = connection.connection();
-                    trace!(
-                        connection_id = connection.id(),
-                        src = %connection.remote_address(),
-                        "{}",
-                        LogMarker::ConnectionReused
-                    );
-                    (Ok(connection.clone()), true)
-                } else {
-                    (
-                        self.endpoint
-                            .connect_to(&recipient.addr())
-                            .and_then(|(connection, connection_incoming)| async move {
-                                recipient.set_connection(connection.clone()).await;
-                                self.connected_peers.insert(connection.clone()).await;
-                                let _ = task::spawn(
-                                    handle_incoming_messages(
-                                        connection.clone(),
-                                        connection_incoming,
-                                        self.event_tx.clone(),
-                                        self.msg_count.clone(),
-                                        self.connected_peers.clone(),
-                                    )
-                                    .in_current_span(),
-                                );
-                                Ok(connection)
-                            })
-                            .await,
-                        false,
-                    )
-                };
-
-                let result = future::ready(connection)
-                    .err_into()
-                    .and_then(|connection| async move {
-                        connection
-                            .send_with(msg_bytes, priority, Some(&retries))
-                            .await
-                    })
-                    .await
-                    .map_err(|err| match err {
-                        qp2p::SendError::ConnectionLost(qp2p::ConnectionError::Closed(
-                            qp2p::Close::Local,
-                        )) => Error::ConnectionClosed,
-                        _ => {
-                            warn!("during sending, received error {:?}", err);
-                            err.into()
-                        }
-                    });
-
-                (result, recipient, reused)
-            }
-            .in_current_span()
-        };
-
         let mut tasks: FuturesUnordered<_> = recipients[0..delivery_group_size]
             .iter()
-            .map(|recipient| send(recipient, msg_bytes.clone(), false))
+            .map(|recipient| self.send_to_one(recipient, msg_id, priority, msg_bytes.clone(), None))
             .collect();
 
         let mut next = delivery_group_size;
         let mut successes = 0;
         let mut failed_recipients = vec![];
 
-        while let Some((result, recipient, reused)) = tasks.next().await {
+        while let Some((recipient, result)) = tasks.next().await {
             match result {
                 Ok(()) => {
                     successes += 1;
                     // count outgoing msgs..
                     self.msg_count.increase_outgoing(recipient.addr());
                 }
-                Err(Error::ConnectionClosed) => {
+                Err(error) if error.is_local_close() => {
                     // The connection was closed by us which means
                     // we are terminating so let's cut this short.
                     return Err(Error::ConnectionClosed);
                 }
-                Err(Error::AddressNotReachable {
-                    err: qp2p::RpcError::Send(qp2p::SendError::ConnectionLost(_)),
-                }) if reused => {
+                Err(SendToOneError::Send {
+                    source: qp2p::SendError::ConnectionLost(_),
+                    connection_id,
+                    reused_connection: true,
+                }) => {
                     // We reused an existing connection, but it was lost when we tried to send. This
                     // could indicate the connection timed out whilst it was held, or some other
                     // transient connection issue. We don't treat this as a failed recipient, and
                     // instead push the same recipient again, but force a reconnection.
-                    tasks.push(send(recipient, msg_bytes.clone(), true));
+                    tasks.push(self.send_to_one(
+                        recipient,
+                        msg_id,
+                        priority,
+                        msg_bytes.clone(),
+                        Some(connection_id),
+                    ));
                 }
-                Err(_) => {
+                Err(error) => {
+                    warn!("during sending, received error {:?}", error);
                     failed_recipients.push(recipient.clone());
 
                     if next < recipients.len() {
-                        tasks.push(send(&recipients[next], msg_bytes.clone(), false));
+                        tasks.push(self.send_to_one(
+                            &recipients[next],
+                            msg_id,
+                            priority,
+                            msg_bytes.clone(),
+                            None,
+                        ));
                         next += 1;
                     }
                 }
@@ -410,6 +312,76 @@ impl Comm {
         }
     }
 
+    // Helper to send a message to a single recipient.
+    #[allow(clippy::needless_lifetimes)] // clippy is wrong here
+    async fn send_to_one<'r>(
+        &self,
+        recipient: &'r Peer,
+        msg_id: MessageId,
+        msg_priority: i32,
+        msg_bytes: Bytes,
+        failed_connection_id: Option<usize>,
+    ) -> (&'r Peer, Result<(), SendToOneError>) {
+        trace!(
+            "Sending message ({} bytes, msg_id: {:?}) to {}",
+            msg_bytes.len(),
+            msg_id,
+            recipient,
+        );
+
+        // TODO: more laid back retries with lower priority, more aggressive with higher
+        let retries = self.back_pressure.get(&recipient.addr()).await;
+
+        let mut reused_connection = true;
+        let connection = recipient
+            .ensure_connection(
+                |connection| Some(connection.id()) != failed_connection_id,
+                |addr| {
+                    reused_connection = false;
+                    async move {
+                        let (connection, connection_incoming) =
+                            self.endpoint.connect_to(&addr).await?;
+                        let _ = task::spawn(
+                            handle_incoming_messages(
+                                connection.clone(),
+                                connection_incoming,
+                                self.event_tx.clone(),
+                                self.msg_count.clone(),
+                            )
+                            .in_current_span(),
+                        );
+                        Ok(connection)
+                    }
+                },
+            )
+            .await
+            .map_err(SendToOneError::Connection);
+
+        if let (true, Ok(connection)) = (reused_connection, &connection) {
+            trace!(
+                connection_id = connection.id(),
+                src = %connection.remote_address(),
+                "{}",
+                LogMarker::ConnectionReused
+            );
+        }
+
+        let result = future::ready(connection)
+            .and_then(|connection| async move {
+                connection
+                    .send_with(msg_bytes, msg_priority, Some(&retries))
+                    .await
+                    .map_err(|source| SendToOneError::Send {
+                        source,
+                        connection_id: connection.id(),
+                        reused_connection,
+                    })
+            })
+            .await;
+
+        (recipient, result)
+    }
+
     /// Regulates comms with the specified peer
     /// according to the cpu load report provided by it.
     pub(crate) async fn regulate(&self, peer: SocketAddr, load_report: LoadReport) {
@@ -429,6 +401,32 @@ impl Comm {
     }
 }
 
+/// Errors that can be returned from `Comm::send_to_one`.
+#[derive(Debug)]
+enum SendToOneError {
+    Connection(qp2p::ConnectionError),
+    Send {
+        source: qp2p::SendError,
+        connection_id: usize,
+        reused_connection: bool,
+    },
+}
+
+impl SendToOneError {
+    fn is_local_close(&self) -> bool {
+        matches!(
+            self,
+            SendToOneError::Connection(qp2p::ConnectionError::Closed(qp2p::Close::Local))
+                | SendToOneError::Send {
+                    source: qp2p::SendError::ConnectionLost(qp2p::ConnectionError::Closed(
+                        qp2p::Close::Local
+                    )),
+                    ..
+                }
+        )
+    }
+}
+
 #[derive(Debug)]
 pub(crate) enum ConnectionEvent {
     Received((UnnamedPeer, Bytes)),
@@ -439,31 +437,26 @@ async fn handle_incoming_connections(
     mut incoming_connections: qp2p::IncomingConnections,
     event_tx: mpsc::Sender<ConnectionEvent>,
     msg_count: MsgCount,
-    connected_peers: ConnectedPeers,
 ) {
     while let Some((connection, connection_incoming)) = incoming_connections.next().await {
-        connected_peers.insert(connection.clone()).await;
-
         let _ = task::spawn(
             handle_incoming_messages(
                 connection,
                 connection_incoming,
                 event_tx.clone(),
                 msg_count.clone(),
-                connected_peers.clone(),
             )
             .in_current_span(),
         );
     }
 }
 
-#[tracing::instrument(skip(incoming_msgs, event_tx, msg_count, connected_peers))]
+#[tracing::instrument(skip(incoming_msgs, event_tx, msg_count))]
 async fn handle_incoming_messages(
     connection: qp2p::Connection,
     mut incoming_msgs: qp2p::ConnectionIncoming,
     event_tx: mpsc::Sender<ConnectionEvent>,
     msg_count: MsgCount,
-    connected_peers: ConnectedPeers,
 ) {
     let connection_id = connection.id();
     let src = connection.remote_address();
@@ -488,9 +481,6 @@ async fn handle_incoming_messages(
         }
     }
 
-    // remove the connection once we notice it end
-    connected_peers.remove_by_address(&src).await;
-
     trace!(%connection_id, %src, "{}", LogMarker::ConnectionClosed);
 }
 
@@ -516,6 +506,7 @@ mod tests {
     use rand::rngs::OsRng;
     use std::{net::Ipv4Addr, time::Duration};
     use tokio::{net::UdpSocket, sync::mpsc, time};
+    use xor_name::XorName;
 
     const TIMEOUT: Duration = Duration::from_secs(1);
 
@@ -738,50 +729,6 @@ mod tests {
         Ok(())
     }
 
-    #[cfg(not(feature = "unstable-no-connection-pooling"))]
-    #[tokio::test(flavor = "multi_thread")]
-    async fn connected_peers() -> Result<()> {
-        let (node_tx, mut node_rx) = mpsc::channel(1);
-        let node_comm = Comm::new(local_addr(), Config::default(), node_tx).await?;
-        let node_addr = node_comm.our_connection_info();
-
-        let (client_tx, mut client_rx) = mpsc::channel(1);
-        let client_comm = Comm::new(local_addr(), Config::default(), client_tx).await?;
-        let client_addr = client_comm.our_connection_info();
-
-        // Establish a connection by sending a message
-        let status = client_comm
-            .send(
-                &[Peer::new(XorName::random(), node_addr)],
-                1,
-                new_test_message()?,
-            )
-            .await?;
-        assert_matches!(status, SendStatus::AllRecipients);
-
-        // We should have recorded the connection
-        assert_matches!(node_rx.recv().await, Some(ConnectionEvent::Received(_)));
-        assert!(
-            node_comm
-                .get_peer_address(&ConnectedPeers::address_to_id(&client_addr))
-                .await
-                .is_some(),
-            "did not find expected connection"
-        );
-
-        // We can reply to the client over the existing connection
-        node_comm
-            .send_on_existing_connection(
-                &[Peer::new(XorName::random(), client_addr)],
-                new_test_message()?,
-            )
-            .await?;
-
-        assert_matches!(client_rx.recv().await, Some(ConnectionEvent::Received(_)));
-
-        Ok(())
-    }
-
     fn new_test_message() -> Result<WireMsg> {
         let dst_location = DstLocation::Node {
             name: XorName::random(),
