@@ -268,12 +268,14 @@ impl Core {
         // if so we can skip the SectionInfo agreement proposal phase.
         if self
             .network_knowledge
-            .set_current_sap(key_share_pk, &sap.prefix())
+            .try_update_current_sap(key_share_pk, &sap.prefix())
             .await
         {
             self.update_self_for_new_node_state_and_fire_events(snapshot)
                 .await
         } else {
+            // This proposal is sent to the current set of elders to be aggregated
+            // and section signed.
             let proposal = Proposal::SectionInfo(sap);
             let recipients: Vec<_> = self
                 .network_knowledge
