@@ -205,7 +205,11 @@ impl NetworkKnowledge {
     /// If we already have the signed SAP and section chain for the provided key and prefix
     /// we make them the current SAP and section chain, and if so, this returns 'true'.
     /// Note this function assumes we already have the key share for the provided section key.
-    pub(super) async fn set_current_sap(&self, section_key: BlsPublicKey, prefix: &Prefix) -> bool {
+    pub(super) async fn try_update_current_sap(
+        &self,
+        section_key: BlsPublicKey,
+        prefix: &Prefix,
+    ) -> bool {
         // Let's try to find the signed SAP corresponding to the provided prefix and section key
         match self.prefix_map.get_signed(prefix) {
             Some(signed_sap) if signed_sap.value.section_key() == section_key => {
@@ -638,7 +642,7 @@ impl NetworkKnowledge {
             });
 
         debug!(
-            ">>>> our size {:?}, theirs {:?}",
+            "Upon section split attempt: our section size {:?}, theirs {:?}",
             our_new_size, sibling_new_size
         );
 
@@ -646,7 +650,6 @@ impl NetworkKnowledge {
         if our_new_size < recommended_section_size()
             || sibling_new_size < recommended_section_size()
         {
-            debug!(">>>> returning here TOO SMALLLLLLLLL hmmmmmm");
             return None;
         }
 
@@ -669,7 +672,6 @@ impl NetworkKnowledge {
         let our_elder_candidates = ElderCandidates::new(our_prefix, our_elders);
         let other_elder_candidates = ElderCandidates::new(other_prefix, other_elders);
 
-        debug!(">>>>> end of split attempt");
         Some((our_elder_candidates, other_elder_candidates))
     }
 }
