@@ -9,11 +9,12 @@
 use std::{
     cmp::Ordering,
     fmt::{self, Display, Formatter},
+    future::Future,
     hash::{Hash, Hasher},
     net::SocketAddr,
     sync::Arc,
 };
-use tokio::sync::RwLock;
+use tokio::sync::{RwLock, RwLockReadGuard};
 use xor_name::{XorName, XOR_NAME_LEN};
 
 /// Network peer identity.
@@ -138,13 +139,114 @@ impl Peer {
         self.connection.read().await.as_ref().cloned()
     }
 
-    /// Set the connection to the peer.
+    /// Copy the connection from another peer, if this peer doesn't have one.
     ///
-    /// If there's an existing connection, it is overwritten and dropped, so ideally this should
-    /// only be performed if the existing connection is known to be lost (we cannot test that
-    /// without using the connection).
-    pub(crate) async fn set_connection(&self, connection: qp2p::Connection) {
-        let _existing = self.connection.write().await.replace(connection);
+    /// This prefers the keep the existing connection, if one is set. This choice is made to avoid
+    /// taking the write lock if we don't need to. [`ensure_connection`] can be used to force a new
+    /// connection to be set, if necessary.
+    pub(crate) async fn merge_connection(&self, other: &Self) {
+        // As a quick sanity check, do nothing if the addresses differ
+        if self.addr != other.addr {
+            return;
+        }
+
+        // Fast-path: try to get a read lock synchronously, and if the connection is set do nothing
+        if let Ok(true) = self
+            .connection
+            .try_read()
+            .map(|connection| connection.is_some())
+        {
+            return;
+        }
+
+        let other_connection = if let Ok(connection) =
+            RwLockReadGuard::try_map(other.connection.read().await, Option::as_ref)
+        {
+            // eager clone to drop the read lock, clones should be quite cheap
+            connection.clone()
+        } else {
+            // There's nothing to do if `other` has no connection
+            return;
+        };
+
+        // Another sanity check: the connection itself matches our address
+        // TODO: we could consider panicking here, since this would represent corrupt state
+        if self.addr != other_connection.remote_address() {
+            return;
+        }
+
+        // Either we couldn't get the read lock, or the connection isn't set, so we have to take the
+        // write lock.
+        let mut guard = self.connection.write().await;
+
+        if guard.is_some() {
+            // The connection was set while we waited, defer to it and do nothing
+            return;
+        }
+
+        // Set the connection
+        *guard = Some(other_connection);
+    }
+
+    /// Ensure the peer has a connection, and connect if not.
+    ///
+    /// This method is tailored to the use-case of connecting on send. In particular, it can be used
+    /// to ensure that `connect` is only called once, even if there are many concurrent calls to
+    /// `ensure_connection`.
+    ///
+    /// `is_valid` is used to determine whether to continue with the existing connection, or whether
+    /// to call `connect` anyway. For example, setting `is_valid = |_| true` would always use any
+    /// existing connection, and so `connect` would only be called once. This mechanism was chosen
+    /// so that, e.g. the connection'd ID could be compared to see if a reconnection had already
+    /// occurred, or force one otherwise (e.g. by setting
+    /// `is_valid = |connection| connection.id() != last_connection_id`).
+    pub(crate) async fn ensure_connection<Connect, Fut>(
+        &self,
+        is_valid: impl Fn(&qp2p::Connection) -> bool,
+        connect: Connect,
+    ) -> Result<RwLockReadGuard<'_, qp2p::Connection>, qp2p::ConnectionError>
+    where
+        Connect: FnOnce(SocketAddr) -> Fut,
+        Fut: Future<Output = Result<qp2p::Connection, qp2p::ConnectionError>>,
+    {
+        // Fast-path: try to get a read lock synchronously, and return the existing connection.
+        if let Some(guard) = self
+            .connection
+            .try_read()
+            .ok()
+            .and_then(|guard| RwLockReadGuard::try_map(guard, Option::as_ref).ok())
+        {
+            if is_valid(&guard) {
+                return Ok(guard);
+            }
+        }
+
+        // If we couldn't get the read lock synchronously, we conservatively take the write lock.
+        // This will prevent anyone else from looking at the connection until we have set one.
+        let mut guard = self.connection.write().await;
+
+        if let Some(connection) = guard.as_ref() {
+            // Someone else set the connection while we waited.
+
+            if is_valid(connection) {
+                // We can't avoid an unwrap here, but we can be sure it will succeed because we hold
+                // the write lock (meaning no one else can fiddle with `self.connection`), and
+                // because we just tested that a connection is set.
+                return Ok(RwLockReadGuard::try_map(guard.downgrade(), Option::as_ref)
+                    .expect("write-locked value can't have changed"));
+            }
+        }
+
+        // We now know the connection isn't set/valid, so we call `connect` and set it.
+        // TODO: we could consider panicking here if `connect` breaks our invariant by returning an
+        // connection with the wrong address.
+        *guard = Some(connect(self.addr).await?);
+
+        // We can't avoid an unwrap here, but we can be sure it will succeed because we hold the
+        // write lock (meaning no one else can fiddle with `self.connection`), and we just set a
+        // connection above.
+        Ok(RwLockReadGuard::try_map(guard.downgrade(), Option::as_ref)
+            .expect("write-locked value can't have changed"))
     }
 }
 
